#!/usr/bin/env node

'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var program = _interopDefault(require('commander'));
var chalk = _interopDefault(require('chalk'));
var fileSize = _interopDefault(require('filesize'));
var path = require('path');
var prettyMs = _interopDefault(require('pretty-ms'));
var readPkg = _interopDefault(require('read-pkg'));
var slash = _interopDefault(require('slash'));
var __ = require('..');
var fs = require('fs');
var rollup = require('rollup');

var name = "bundlib";
var displayName = "Bundlib";
var version = "0.14.3";

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function oneByOne(arr, callback) {
  var index = 0;
  var len = arr.length;
  var last = len - 1;

  var next = function next() {
    if (index > last) {
      return;
    }

    callback(arr[index], index < last ? next : null);
    index++;
  };

  next();
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function build(configs, callbacks) {
  if (callbacks.start) {
    callbacks.start();
  }

  var cache = {};
  oneByOne(configs, function (config, next) {
    try {
      var _temp3 = function _temp3(_result) {
        if (_exit2) return _result;

        if (next) {
          return next();
        }

        if (callbacks.end) {
          callbacks.end();
        }
      };

      var _exit2 = false;
      var input = config.input,
          output = config.output;
      var outputFile = output.file,
          format = output.format;
      var isModuleBuild = format === 'cjs' || format === 'es';
      var cacheKey = (isModuleBuild ? 'module' : 'browser') + ":" + input;
      config.cache = cache[cacheKey];

      if (callbacks.buildStart) {
        callbacks.buildStart(input, outputFile);
      }

      var _temp4 = _catch(function () {
        var currentTime = Date.now();
        return Promise.resolve(rollup.rollup(config)).then(function (buildResult) {
          config.cache = cache[cacheKey] = buildResult.cache;
          return Promise.resolve(buildResult.write(output)).then(function () {
            var _statSync = fs.statSync(outputFile),
                size = _statSync.size;

            var totalTime = Date.now() - currentTime;

            if (callbacks.buildEnd) {
              callbacks.buildEnd(outputFile, size, totalTime);
            }
          });
        });
      }, function (err) {
        if (callbacks.error) {
          callbacks.error(err);
        }

        if (callbacks.end) {
          callbacks.end();
        }

        throw err;
      });

      return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(_temp3) : _temp3(_temp4));
    } catch (e) {
      return Promise.reject(e);
    }
  });
}

function watch(configs, callbacks) {
  var start = callbacks.start,
      end = callbacks.end,
      buildStart = callbacks.buildStart,
      buildEnd = callbacks.buildEnd,
      error = callbacks.error;

  var ERROR = error && function (event) {
    error(event.error);
  };

  var table = {
    START: start && function () {
      start();
    },
    END: end && function () {
      end();
    },
    BUNDLE_START: buildStart && function (event) {
      var out = event.output;
      var len = out.length;

      for (var i = 0; i < len; i++) {
        buildStart(event.input, out[i]);
      }
    },
    BUNDLE_END: buildEnd && function (event) {
      var out = event.output;
      var len = out.length;

      for (var i = 0; i < len; i++) {
        var stats = fs.statSync(out[i]);
        buildEnd(out[i], stats.size, event.duration || 0);
      }
    },
    ERROR: ERROR,
    FATAL: ERROR
  };
  var watcher = rollup.watch(configs);
  watcher.on('event', function (event) {
    var code = event.code;
    var handle = table[code];

    if (handle) {
      handle(event);
    }
  });
}

var bundlib = function bundlib(cwd, options, callbacks, pkg) {
  try {
    return Promise.resolve(__.configsFromPkg(cwd, options, pkg)).then(function (configs) {
      var warn = callbacks.warn;
      var onwarn = warn ? function (warning) {
        return warn(warning);
      } : function () {
        return null;
      };
      configs.forEach(function (config) {
        config.onwarn = onwarn;
      });
      (options.watch ? watch : build)(configs, callbacks);
    });
  } catch (e) {
    return Promise.reject(e);
  }
};

function create(name) {
  var method = console[name];
  return function (msg) {
    return method(chalk.cyan(msg));
  };
}

var log = create('log');
var error = create('error');
function logError(err) {
  if (err.stack) {
    error(err.stack);
  }

  var tag = chalk.bold.bgWhite.red.inverse(' error ');
  error(tag + " " + (err.message || err));
}

function _catch$1(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

var action = function action(displayName, version, silent, options) {
  try {
    var _temp3 = function _temp3() {
      var buildIndex = 0;
      var callbacks = {
        error: function error(err) {
          logError(err);
        }
      };

      if (!silent) {
        _extends(callbacks, {
          buildEnd: function buildEnd(filename, size, duration) {
            var tag = green.bgBlack.inverse(' built ');
            var path$1 = yellow(slash(path.relative(cwd, filename)));
            var colorSize = magenta(fileSize(size));
            var colorTime = magenta(prettyMs(duration, {
              secondsDecimalDigits: 2
            }));
            log(tag + " " + path$1 + " ( " + colorSize + " in " + colorTime + " )");
          },
          error: function error(err) {
            logError(err);
          },
          warn: function warn(warning) {
            var message = warning;

            if (typeof message === 'object') {
              var _message = message,
                  plugin = _message.plugin,
                  msg = _message.message;
              message = msg;

              if (plugin) {
                message = "(plugin " + magenta(plugin) + ") " + message;
              }
            }

            var tag = magenta('warning:');
            log(tag + " " + message);
          }
        });

        if (options.watch) {
          _extends(callbacks, {
            start: function start() {
              if (buildIndex) {
                log("rebuilding...\n");
              }

              buildIndex++;
            },
            end: function end() {
              log("\nwaiting for changes...");
            }
          });
        }
      }

      return _catch$1(function () {
        return Promise.resolve(bundlib(cwd, options, callbacks, pkg)).then(function () {});
      }, function (err) {
        logError(err);
        throw err;
      });
    };

    var cwd = process.cwd();
    var pkg;

    var _temp4 = function () {
      if (!silent) {
        var appInfo = prjInfo(displayName, version);
        log(appInfo + "\n");
        var filename = yellow('package.json');
        log("reading: " + filename);
        return Promise.resolve(readPkg({
          cwd: cwd,
          normalize: false
        })).then(function (_readPkg) {
          pkg = _readPkg;
          var _pkg = pkg,
              prjName = _pkg.name,
              prjDispName = _pkg.displayName,
              prjVer = _pkg.version;
          var prjInfoName = prjDispName || prjName;

          if (prjInfoName && prjVer) {
            var info = prjInfo(prjInfoName, prjVer);
            log("building: " + info + "\n");
          }
        });
      }
    }();

    return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(_temp3) : _temp3(_temp4));
  } catch (e) {
    return Promise.reject(e);
  }
};
var bold = chalk.bold;
var green = bold.green;
var yellow = bold.yellow;
var magenta = bold.magenta;

function prjInfo(name, ver) {
  var projName = green(name);
  var projVer = yellow("v" + ver);
  return projName + " " + projVer;
}

if (!chalk.level && 'MINGW_CHOST' in process.env) {
  chalk.level = 1;
}

program.version(version, '-v, --version').name(name).option('-d, --dev', 'create development builds').option('-w, --watch', 'run bundlib in watch mode').option('-s, --silent', 'prevent messages from showing in the console').action(function () {
  var dev = program.dev,
      watch = program.watch,
      silent = program.silent;
  action(displayName, version, silent, {
    dev: dev,
    watch: watch
  })["catch"](function (err) {
    throw err;
  });
}).parse(process.argv);
//# sourceMappingURL=bundlib-cli.js.map
